%Case 1 result generation: LSE using perfect data (CL, CD, CC, Cl, Cm, Cn)
clear c1;
c1.t = CL.time;
c1.CL = CL.signals.values;
c1.CD = CD.signals.values;
c1.CC = CC.signals.values;
c1.Cl = Cl.signals.values;
c1.Cm = Cm.signals.values;
c1.Cn = Cn.signals.values;
c1.alpha = alpha.signals.values;
c1.beta = beta.signals.values;
c1.p = w_body.signals.values(:, 1);
c1.q = w_body.signals.values(:, 2);
c1.r = w_body.signals.values(:, 3);
c1.delta_a = delta.signals.values(:, 1);
c1.delta_e = delta.signals.values(:, 2);
c1.delta_r = delta.signals.values(:, 3);

s = c1;

%LS estimation:
disp('Lift CL[1, alpha]');
[c1.CL_coeff c1.CL_phi] = lse(s.CL, [s.alpha]);
disp('Drag CD[1, abs(alpha), abs(beta)]');
[c1.CD_coeff c1.CD_phi] = lse(s.CD, [abs(s.alpha), abs(s.beta)]);
disp('Side CY[1, alpha, beta]');
[c1.CC_coeff c1.CC_phi] = lse(s.CC, [s.alpha, s.beta]);
disp('Roll Cl[1, beta, p, delta_a]');
[c1.Cl_coeff c1.Cl_phi] = lse(s.Cl, [s.beta, s.p, s.delta_a]);
disp('Pitch Cm[1, alpha, q, delta_e]');
[c1.Cm_coeff c1.Cm_phi] = lse(s.Cm, [s.alpha, s.q s.delta_e]);
disp('Yaw Cn[1, beta, r, delta_r]');
[c1.Cn_coeff c1.Cn_phi] = lse(s.Cn, [s.beta, s.r, s.delta_r]);

%Plotting:

%Plot aerodynamical forces:
figure(1);
clf;
subplot(3, 1, 1);
h = plot(s.t, c1.CD_phi*c1.CD_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(s.t, s.CD, 'r');
set(h, 'LineWidth', 1);
ylabel('C_D');
legend('Fitted', 'True', 'location', 'SouthEast', 'Orientation', 'horizontal');
title('Fitted aerodynamical forces using LS and perfect data', 'FontSize', 14);

subplot(3, 1, 2);
h = plot(s.t, c1.CC_phi*c1.CC_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(s.t, s.CC, 'r');
set(h, 'LineWidth', 1);
ylabel('C_C');

subplot(3, 1, 3);
h = plot(s.t, c1.CL_phi*c1.CL_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(s.t, s.CL, 'r');
set(h, 'LineWidth', 1);
ylabel('C_L');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c1_aero_f.pdf');


%Plot aerodynamical moments:
figure(2);
clf;
subplot(3, 1, 1);
h = plot(s.t, c1.Cl_phi*c1.Cl_coeff, 'b--');
title('Fitted aerodynamical moments using LS and perfect data', 'FontSize', 14);
set(h, 'LineWidth', 3);
hold on;
h = plot(s.t, s.Cl, 'r');
set(h, 'LineWidth', 1);
ylabel('C_l');
legend('Fitted', 'True', 'location', 'NorthEast', 'Orientation', 'horizontal');

subplot(3, 1, 2);
h = plot(s.t, c1.Cm_phi*c1.Cm_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(s.t, s.Cm, 'r');
set(h, 'LineWidth', 1);
ylabel('C_m');

subplot(3, 1, 3);
h = plot(s.t, c1.Cn_phi*c1.Cn_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(s.t, s.Cn, 'r');
set(h, 'LineWidth', 1);
ylabel('C_n');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c1_aero_m.pdf');

%Plot input sequence:
figure(3);
clf;
subplot(3, 1, 1);
h = plot(s.t, s.delta_a*180.0/pi, 'b');
title('Input sequence during simulation', 'FontSize', 14);
set(h, 'LineWidth', 2);
ylabel('\delta_a (deg)');
grid on;

subplot(3, 1, 2);
h = plot(s.t, s.delta_e*180.0/pi, 'b');
set(h, 'LineWidth', 2);
ylabel('\delta_e (deg)');
grid on;

subplot(3, 1, 3);
h = plot(s.t, s.delta_r*180.0/pi, 'b');
set(h, 'LineWidth', 2);
ylabel('\delta_r (deg)');
xlabel('Time (s)');
grid on;

print('-dpdf', '../report/gfx/c1_input.pdf');

%Plot wind angles:
figure(4);
clf;
subplot(3, 1, 1);
h = plot(s.t, s.alpha*180.0/pi, 'b--');
title('Actual wind angles during simulation', 'FontSize', 14);
set(h, 'LineWidth', 1);
ylabel('\alpha (deg)');
grid on;

subplot(3, 1, 2);
h = plot(s.t, s.beta*180.0/pi, 'b--');
set(h, 'LineWidth', 1);
ylabel('\beta (deg)');
grid on;

%Plot pqr:
subplot(3, 1, 3);
h = plot(s.t, [s.p s.q s.r]*180.0/pi);
title('Rotation rates during simulation', 'FontSize', 14);
set(h, 'LineWidth', 2);
grid on;
legend('p','q','r', 'Orientation', 'horizontal');
ylabel('Rotation rate (deg/s)');
xlabel('Time (s)');
ylim([-350, 350]);

print('-dpdf', '../report/gfx/c1_windrot.pdf');