%Create sensor signals to use in calculation:
c2.t = sensor_a.time;
t = c2.t;

%Dynamic pressure sensor (pitot-static-tube):
c2.q_bar = sensor_q_bar.signals.values;

%Accelerometer:
c2.ax = sensor_a.signals.values(:, 1);
c2.ay = sensor_a.signals.values(:, 2);
c2.az = sensor_a.signals.values(:, 3);

%Angular velocity (gyro):
c2.p = sensor_omega.signals.values(:, 1);
c2.q = sensor_omega.signals.values(:, 2);
c2.r = sensor_omega.signals.values(:, 3);

%Magnetic field sensor:
c2.mx = sensor_m.signals.values(:, 1);
c2.my = sensor_m.signals.values(:, 2);
c2.mz = sensor_m.signals.values(:, 3);

%Delta controls:
c2.delta_a = sensor_delta.signals.values(:, 1);
c2.delta_e = sensor_delta.signals.values(:, 2);
c2.delta_r = sensor_delta.signals.values(:, 3);

%Angular acceleration:
c2.dp = mpoly(c2.p, t, 1, 1, 1, 1, t);
c2.dq = mpoly(c2.q, t, 1, 1, 1, 1, t);
c2.dr = mpoly(c2.r, t, 1, 1, 1, 1, t);
c2.dp_direct = [0; diff(c2.p) ./ diff(t)];
c2.dq_direct = [0; diff(c2.q) ./ diff(t)];
c2.dr_direct = [0; diff(c2.r) ./ diff(t)];
%c2.dp = c2.dp_direct;
%c2.dq = c2.dq_direct;
%c2.dr = c2.dr_direct;

%Wind vanes:
c2.alpha_E = sensor_alpha_E.signals.values;
c2.flank_E = sensor_flank_E.signals.values;

%Correct alpha:
c2.alpha_C = c2.alpha_E + c2.q*alpha_vane_position(1)./sqrt(c2.q_bar/1.2/0.5) - c2.p*alpha_vane_position(2)./sqrt(c2.q_bar/1.2/0.5);
c2.alpha = c2.alpha_C;

%Find beta from flank:
c2.beta_E = tan(atan(c2.flank.*cos(c2.alpha)));
c2.beta_C = c2.flank_E - c2.r*beta_vane_position(1)./sqrt(c2.q_bar/1.2/0.5) + c2.p*beta_vane_position(3)./sqrt(c2.q_bar/1.2/0.5);
c2.beta = c2.beta_C;

%Calculate body-axis coefficients:
c2.CX_mes = (m*c2.ax(:) - T) ./ (c2.q_bar*S);
c2.CY_mes = m*c2.ay(:) ./ (c2.q_bar*S);
c2.CZ_mes = m*c2.az(:) ./ (c2.q_bar*S);
%Find stability axis coefficients:
c2.CDs_mes = c2.CX_mes .* cos(c2.alpha) + c2.CZ_mes .*sin(c2.alpha);
c2.CYs_mes = c2.CY_mes;
c2.CLs_mes = c2.CZ_mes .* cos(c2.alpha) - c2.CX_mes .*sin(c2.alpha);
%Find wind axis coefficients:
c2.CDw_mes = c2.CDs_mes .* cos(c2.beta) + c2.CYs_mes .* sin(c2.beta);
c2.CYw_mes = c2.CYs_mes .* cos(c2.beta) - c2.CDs_mes .* sin(c2.beta);
c2.CLw_mes = c2.CLs_mes;
%Calculate angular coefficients:
c2.Cl_mes = (Jx*c2.dp - Jxz*(c2.p.*c2.q + c2.dr) + (Jz-Jy)*c2.q.*c2.r) ./ (c2.q_bar*S*b);
c2.Cm_mes = (Jy*c2.dq + (Jx-Jz)*c2.p.*c2.r + Jxz*(c2.p.*c2.p - c2.r.*c2.r)) ./ (c2.q_bar*S*c);
c2.Cn_mes = (Jz*c2.dr - Jxz*(c2.dp - c2.q.*c2.r) + (Jy-Jx)*c2.p.*c2.q) ./ (c2.q_bar*S*b);

%Find least square estimates:
[c2.CL_coeff c2.CL_phi] = lse(c2.CLw_mes, [c2.alpha]);
[c2.CD_coeff c2.CD_phi] = lse(c2.CDw_mes, [abs(c2.alpha), abs(c2.beta)]);
[c2.CC_coeff c2.CC_phi] = lse(c2.CYw_mes, [c2.beta]);
[c2.Cl_coeff c2.Cl_phi] = lse(c2.Cl_mes, [c2.beta, c2.p, c2.delta_a]);
[c2.Cm_coeff c2.Cm_phi] = lse(c2.Cm_mes, [c2.alpha, c2.q, c2.delta_e]);
[c2.Cn_coeff c2.Cn_phi] = lse(c2.Cn_mes, [c2.beta, c2.r, c2.delta_r]);

%Calculate COD:
c2.cod(1) = cod(c2.CDw_mes, c2.CD_phi*c2.CD_coeff);
c2.cod(2) = cod(c2.CYw_mes, c2.CC_phi*c2.CC_coeff);
c2.cod(3) = cod(c2.CLw_mes, c2.CL_phi*c2.CL_coeff);
c2.cod(4) = cod(c2.Cl_mes, c2.Cl_phi*c2.Cl_coeff);
c2.cod(5) = cod(c2.Cm_mes, c2.Cm_phi*c2.Cm_coeff);
c2.cod(6) = cod(c2.Cn_mes, c2.Cn_phi*c2.Cn_coeff);

%Plotting:

%Plot wind angles:
figure(3);
clf;
subplot(2, 1, 1);
h = plot(alpha.time, alpha.signals.values*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c2.t, c2.alpha_E*180.0/pi, 'r--');
set(h, 'LineWidth', 1);
h = plot(c2.t, c2.alpha_C*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('\alpha (deg)');
legend('True', 'Measured', 'Corrected');
title('Actual, measured and corrected wind angles during simulation', 'FontSize', 14);
grid on;

subplot(2, 1, 2);
h = plot(beta.time, beta.signals.values*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c2.t, c2.flank_E*180.0/pi, 'r--');
set(h, 'LineWidth', 1);
h = plot(c2.t, c2.beta_C*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('\beta (deg)');
grid on;
print('-dpdf', '../report/gfx/c2_wind.pdf');

%Plot aerodynamical forces:
figure(1);
clf;
subplot(3, 1, 1);
title('Calculated, fitted and true aerodynamical forces', 'FontSize', 14);
h = plot(c2.t, c2.CD_phi*c2.CD_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(CD.time, CD.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c2.t, c2.CDw_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_D');
legend('Fitted', 'True', 'Calculated', 'location', 'SouthEast', 'Orientation', 'horizontal');

subplot(3, 1, 2);
h = plot(c2.t, c2.CC_phi*c2.CC_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(CC.time, CC.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c2.t, c2.CYw_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_C');

subplot(3, 1, 3);
h = plot(c2.t, c2.CL_phi*c2.CL_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(CL.time, CL.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c2.t, c2.CLw_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_L');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c2_aero_f.pdf');


%Plot aerodynamical moments:
figure(2);
clf;
subplot(3, 1, 1);
h = plot(c2.t, c2.Cl_phi*c2.Cl_coeff, 'b--');
title('Fitted, calculated and true aerodynamical moments', 'FontSize', 14);
set(h, 'LineWidth', 3);
hold on;
h = plot(Cl.time, Cl.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c2.t, c2.Cl_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_l');
legend('Fitted', 'True', 'Calculated', 'location', 'NorthEast', 'Orientation', 'horizontal');

subplot(3, 1, 2);
h = plot(c2.t, c2.Cm_phi*c2.Cm_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(Cm.time, Cm.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c2.t, c2.Cm_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_m');

subplot(3, 1, 3);
h = plot(c2.t, c2.Cn_phi*c2.Cn_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(Cn.time, Cn.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c2.t, c2.Cn_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_n');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c2_aero_m.pdf');

%Plot angular acceleration:
figure(5);
clf;
subplot(3, 1, 1);
title('Angular acceleration during simulation', 'FontSize', 14);
h = plot(w_body_dot.time, w_body_dot.signals.values(:, 1)*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c2.t, c2.dp*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('p_dot (deg/s^2)');

subplot(3, 1, 2);
h = plot(w_body_dot.time, w_body_dot.signals.values(:, 2)*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c2.t, c2.dq*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('r_dot (deg/s^2)');

subplot(3, 1, 3);
h = plot(w_body_dot.time, w_body_dot.signals.values(:, 3)*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c2.t, c2.dr*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('r_dot (deg/s^2)');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c2_rot_acc.pdf');

%Print table of results:
fprintf('Table of results:\n');
fprintf('\t$C_{D_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_0, c2.CD_coeff(1), (CD_0-c2.CD_coeff(1)) / CD_0 * 100);
fprintf('\t$C_{D_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_alpha, c2.CD_coeff(2), (CD_alpha-c2.CD_coeff(2)) / CD_alpha * 100);
fprintf('\t$C_{D_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_beta, c2.CD_coeff(3), (CD_beta-c2.CD_coeff(3)) / CD_beta * 100);
fprintf('\t$C_{C_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CC_0, c2.CC_coeff(1), (CC_0-c2.CC_coeff(1)) / CC_0 * 100);
fprintf('\t$C_{C_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CC_beta, c2.CC_coeff(2), (CC_beta-c2.CC_coeff(2)) / CC_beta * 100);
fprintf('\t$C_{L_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CL_0, c2.CL_coeff(1), (CL_0-c2.CL_coeff(1)) / CL_0 * 100);
fprintf('\t$C_{L_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CL_alpha, c2.CL_coeff(2), (CL_alpha-c2.CL_coeff(2)) / CL_alpha * 100);
fprintf('\t$C_{l_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_0, c2.Cl_coeff(1), (Cl_0-c2.Cl_coeff(1)) / Cl_0 * 100);
fprintf('\t$C_{l_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_beta, c2.Cl_coeff(2), (Cl_beta-c2.Cl_coeff(2)) / Cl_beta * 100);
fprintf('\t$C_{l_{p}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_p, c2.Cl_coeff(3), (Cl_p-c2.Cl_coeff(3)) / Cl_p * 100);
fprintf('\t$C_{l_{\\delta_a}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_delta_a, c2.Cl_coeff(4), (Cl_delta_a-c2.Cl_coeff(4)) / Cl_delta_a * 100);
fprintf('\t$C_{m_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_0, c2.Cm_coeff(1), (Cm_0-c2.Cm_coeff(1)) / Cm_0 * 100);
fprintf('\t$C_{m_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_alpha, c2.Cm_coeff(2), (Cm_alpha-c2.Cm_coeff(2)) / Cm_alpha * 100);
fprintf('\t$C_{m_{q}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_q, c2.Cm_coeff(3), (Cm_q-c2.Cm_coeff(3)) / Cm_q * 100);
fprintf('\t$C_{m_{\\delta_e}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_delta_e, c2.Cm_coeff(4), (Cm_delta_e-c2.Cm_coeff(4)) / Cm_delta_e * 100);
fprintf('\t$C_{n_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_0, c2.Cn_coeff(1), (Cn_0-c2.Cn_coeff(1)) / Cn_0 * 100);
fprintf('\t$C_{n_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_beta, c2.Cn_coeff(2), (Cn_beta-c2.Cn_coeff(2)) / Cn_beta * 100);
fprintf('\t$C_{n_{r}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_r, c2.Cn_coeff(3), (Cn_r-c2.Cn_coeff(3)) / Cn_r * 100);
fprintf('\t$C_{n_{\\delta_r}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_delta_r, c2.Cn_coeff(4), (Cn_delta_r-c2.Cn_coeff(4)) / Cn_delta_r * 100);

%Print coefficients of determination:
fprintf('Coefficients of determination:\n');
fprintf('\t%.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%%\\\\\n', c2.cod(1), c2.cod(2), c2.cod(3), c2.cod(4), c2.cod(5), c2.cod(6));


