%Create sensor signals to use in calculation:
c3.t = sensor_a.time;
t = c3.t;

%Dynamic pressure sensor (pitot-static-tube):
c3.q_bar = sensor_q_bar.signals.values;

%Accelerometer:
c3.ax = sensor_a.signals.values(:, 1);
c3.ay = sensor_a.signals.values(:, 2);
c3.az = sensor_a.signals.values(:, 3);

%Angular velocity (gyro):
c3.p = sensor_omega.signals.values(:, 1);
c3.q = sensor_omega.signals.values(:, 2);
c3.r = sensor_omega.signals.values(:, 3);

%Magnetic field sensor:
c3.mx = sensor_m.signals.values(:, 1);
c3.my = sensor_m.signals.values(:, 2);
c3.mz = sensor_m.signals.values(:, 3);

%Delta controls:
c3.delta_a = sensor_delta.signals.values(:, 1);
c3.delta_e = sensor_delta.signals.values(:, 2);
c3.delta_r = sensor_delta.signals.values(:, 3);

%Angular acceleration:
c3.dp = mpoly(c3.p, t, 1, 1, 1, 1, t);
c3.dq = mpoly(c3.q, t, 2, 5, 5, 1, t);
c3.dr = mpoly(c3.r, t, 1, 1, 1, 1, t);

%c3.dp_direct = [0; diff(c3.p) ./ diff(t)];
%c3.dq_direct = [0; diff(c3.q) ./ diff(t)];
%c3.dr_direct = [0; diff(c3.r) ./ diff(t)];
%c3.dp = c3.dp_direct;
%c3.dq = c3.dq_direct;
%c3.dr = c3.dr_direct;

%Wind vanes:
c3.alpha_E = sensor_alpha_E.signals.values;
c3.flank_E = sensor_flank_E.signals.values;

%Correct alpha:
c3.alpha_C = c3.alpha_E + c3.q*alpha_vane_position(1)./sqrt(c3.q_bar/1.2/0.5) - c3.p*alpha_vane_position(2)./sqrt(c3.q_bar/1.2/0.5);
c3.alpha = c3.alpha_C;

%Find beta from flank:
c3.beta_from_flank = tan(atan(c3.flank_E.*cos(c3.alpha)));
c3.beta_C = c3.flank_E - c3.r*beta_vane_position(1)./sqrt(c3.q_bar/1.2/0.5) + c3.p*beta_vane_position(3)./sqrt(c3.q_bar/1.2/0.5);
c3.beta = c3.beta_C;

%Calculate body-axis coefficients:
c3.CX_mes = (m*c3.ax(:) - T) ./ (c3.q_bar*S);
c3.CY_mes = m*c3.ay(:) ./ (c3.q_bar*S);
c3.CZ_mes = m*c3.az(:) ./ (c3.q_bar*S);
%Find stability axis coefficients:
c3.CDs_mes = c3.CX_mes .* cos(c3.alpha) + c3.CZ_mes .*sin(c3.alpha);
c3.CYs_mes = c3.CY_mes;
c3.CLs_mes = c3.CZ_mes .* cos(c3.alpha) - c3.CX_mes .*sin(c3.alpha);
%Find wind axis coefficients:
c3.CDw_mes = c3.CDs_mes .* cos(c3.beta) + c3.CYs_mes .* sin(c3.beta);
c3.CYw_mes = c3.CYs_mes .* cos(c3.beta) - c3.CDs_mes .* sin(c3.beta);
c3.CLw_mes = c3.CLs_mes;
%Calculate angular coefficients:
c3.Cl_mes = (Jx*c3.dp - Jxz*(c3.p.*c3.q + c3.dr) + (Jz-Jy)*c3.q.*c3.r) ./ (c3.q_bar*S*b);
c3.Cm_mes = (Jy*c3.dq + (Jx-Jz)*c3.p.*c3.r + Jxz*(c3.p.*c3.p - c3.r.*c3.r)) ./ (c3.q_bar*S*c);
c3.Cn_mes = (Jz*c3.dr - Jxz*(c3.dp - c3.q.*c3.r) + (Jy-Jx)*c3.p.*c3.q) ./ (c3.q_bar*S*b);

%Find least square estimates:
[c3.CL_coeff c3.CL_phi c3.CL_res] = lse(c3.CLw_mes, [c3.alpha]);
[c3.CD_coeff c3.CD_phi c3.CD_res] = lse(c3.CDw_mes, [abs(c3.alpha), abs(c3.beta)]);
[c3.CC_coeff c3.CC_phi c3.CC_res] = lse(c3.CYw_mes, [c3.beta]);
[c3.Cl_coeff c3.Cl_phi c3.Cl_res] = lse(c3.Cl_mes, [c3.beta, c3.p, c3.delta_a]);
[c3.Cm_coeff c3.Cm_phi c3.Cm_res] = lse(c3.Cm_mes, [c3.alpha, c3.q, c3.delta_e]);
[c3.Cn_coeff c3.Cn_phi c3.Cn_res] = lse(c3.Cn_mes, [c3.beta, c3.r, c3.delta_r]);

%Find least square convergence time series:
c3.CL_coeffs = lse_conv(c3.CLw_mes, [c3.alpha], 1);

%Calculate cod:
c3.cod(1) = cod(c3.CDw_mes, c3.CD_phi*c3.CD_coeff);
c3.cod(2) = cod(c3.CYw_mes, c3.CC_phi*c3.CC_coeff);
c3.cod(3) = cod(c3.CLw_mes, c3.CL_phi*c3.CL_coeff);
c3.cod(4) = cod(c3.Cl_mes, c3.Cl_phi*c3.Cl_coeff);
c3.cod(5) = cod(c3.Cm_mes, c3.Cm_phi*c3.Cm_coeff);
c3.cod(6) = cod(c3.Cn_mes, c3.Cn_phi*c3.Cn_coeff);

%Plotting:

%Plot aerodynamical forces:
figure(1);
clf;
subplot(3, 1, 1);
title('Calculated, fitted and true aerodynamical forces', 'FontSize', 14);
h = plot(c3.t, c3.CDw_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c3.t, c3.CD_phi*c3.CD_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(CD.time, CD.signals.values, 'g--');
set(h, 'LineWidth', 2);
ylabel('C_D');
legend('Calculated','Fitted','True', 'location', 'SouthEast', 'Orientation', 'horizontal');

subplot(3, 1, 2);
h = plot(c3.t, c3.CYw_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c3.t, c3.CC_phi*c3.CC_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(CC.time, CC.signals.values, 'g--');
set(h, 'LineWidth', 2);

ylabel('C_C');

subplot(3, 1, 3);
h = plot(c3.t, c3.CLw_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c3.t, c3.CL_phi*c3.CL_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(CL.time, CL.signals.values, 'g--');
set(h, 'LineWidth', 2);

ylabel('C_L');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c3_aero_f.pdf');


%Plot aerodynamical moments:
figure(2);
clf;
subplot(3, 1, 1);
h = plot(c3.t, c3.Cl_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c3.t, c3.Cl_phi*c3.Cl_coeff, 'b--');
title('Fitted, calculated and true aerodynamical moments', 'FontSize', 14);
set(h, 'LineWidth', 3);
h = plot(Cl.time, Cl.signals.values, 'g--');
set(h, 'LineWidth', 2);

ylabel('C_l');
legend('Calculated','Fitted', 'True', 'location', 'NorthEast', 'Orientation', 'horizontal');

subplot(3, 1, 2);
h = plot(c3.t, c3.Cm_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c3.t, c3.Cm_phi*c3.Cm_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(Cm.time, Cm.signals.values, 'g--');
set(h, 'LineWidth', 2);

ylabel('C_m');

subplot(3, 1, 3);
h = plot(c3.t, c3.Cn_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c3.t, c3.Cn_phi*c3.Cn_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(Cn.time, Cn.signals.values, 'g--');
set(h, 'LineWidth', 2);

ylabel('C_n');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c3_aero_m.pdf');

%Plot wind angles:
figure(4);
clf;
subplot(2, 1, 1);
h = plot(c3.t, c3.alpha*180.0/pi, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(alpha.time, alpha.signals.values*180.0/pi, 'b--');
title('Actual and measured wind angles during simulation', 'FontSize', 14);
set(h, 'LineWidth', 2);
ylabel('\alpha (deg)');
legend('Measured', 'True');
grid on;

subplot(2, 1, 2);
h = plot(c3.t, c3.beta*180.0/pi, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(beta.time, beta.signals.values*180.0/pi, 'b--');
set(h, 'LineWidth', 2);
ylabel('\beta (deg)');
grid on;
print('-dpdf', '../report/gfx/c3_wind.pdf');

%Plot angular acceleration:
figure(5);
clf;
subplot(3, 1, 1);
title('Angular acceleration during simulation', 'FontSize', 14);
h = plot(w_body_dot.time, w_body_dot.signals.values(:, 1)*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c3.t, c3.dp*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('p (deg/s^2)');

subplot(3, 1, 2);
h = plot(w_body_dot.time, w_body_dot.signals.values(:, 2)*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c3.t, c3.dq*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('q (deg/s^2)');

subplot(3, 1, 3);
h = plot(w_body_dot.time, w_body_dot.signals.values(:, 3)*180.0/pi, 'c');
set(h, 'LineWidth', 2);
hold on;
h = plot(c3.t, c3.dr*180.0/pi, 'k--');
set(h, 'LineWidth', 1);
ylabel('r (deg/s^2)');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c3_rot_acc.pdf');

%Plot dynamic pressure:
figure(10);
clf;
title('Measured dynamic pressure during simulation', 'FontSize', 14);
h = plot(c3.t, c3.q_bar, 'k');
set(h, 'LineWidth', 1);
hold on;
h = plot(sim_q_bar.time, sim_q_bar.signals.values, 'c');
set(h, 'LineWidth', 2);

grid on;
ylabel('Pressure (Pa/m^2)');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c3_pressure.pdf');

%Plot parameter convergence:
figure(11);
clf;
h = plot(c3.t(1:length(c3.t)), c3.CL_coeffs(1:length(c3.CL_coeffs), 2), '--');
title('Estimation of C_{L_{\alpha}}', 'FontSize', 14);
hold on;
plot(c3.t, CL_alpha*ones(1,length(c3.t)), 'c');
ylim([-1.0 -0.4])
grid on;
set(h, 'LineWidth', 1);
ylabel('C_{L_{\alpha}}');
xlabel('Time (s)');
legend('Estimation', 'True');
print('-dpdf', '../report/gfx/c3_cl_conv.pdf');

%Print table of results:
fprintf('Table of results:\n');
fprintf('\t$C_{D_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_0, c3.CD_coeff(1), (CD_0-c3.CD_coeff(1)) / CD_0 * 100);
fprintf('\t$C_{D_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_alpha, c3.CD_coeff(2), (CD_alpha-c3.CD_coeff(2)) / CD_alpha * 100);
fprintf('\t$C_{D_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_beta, c3.CD_coeff(3), (CD_beta-c3.CD_coeff(3)) / CD_beta * 100);
fprintf('\t$C_{C_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CC_0, c3.CC_coeff(1), (CC_0-c3.CC_coeff(1)) / CC_0 * 100);
fprintf('\t$C_{C_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CC_beta, c3.CC_coeff(2), (CC_beta-c3.CC_coeff(2)) / CC_beta * 100);
fprintf('\t$C_{L_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CL_0, c3.CL_coeff(1), (CL_0-c3.CL_coeff(1)) / CL_0 * 100);
fprintf('\t$C_{L_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CL_alpha, c3.CL_coeff(2), (CL_alpha-c3.CL_coeff(2)) / CL_alpha * 100);
fprintf('\t$C_{l_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_0, c3.Cl_coeff(1), (Cl_0-c3.Cl_coeff(1)) / Cl_0 * 100);
fprintf('\t$C_{l_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_beta, c3.Cl_coeff(2), (Cl_beta-c3.Cl_coeff(2)) / Cl_beta * 100);
fprintf('\t$C_{l_{p}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_p, c3.Cl_coeff(3), (Cl_p-c3.Cl_coeff(3)) / Cl_p * 100);
fprintf('\t$C_{l_{\\delta_a}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_delta_a, c3.Cl_coeff(4), (Cl_delta_a-c3.Cl_coeff(4)) / Cl_delta_a * 100);
fprintf('\t$C_{m_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_0, c3.Cm_coeff(1), (Cm_0-c3.Cm_coeff(1)) / Cm_0 * 100);
fprintf('\t$C_{m_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_alpha, c3.Cm_coeff(2), (Cm_alpha-c3.Cm_coeff(2)) / Cm_alpha * 100);
fprintf('\t$C_{m_{q}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_q, c3.Cm_coeff(3), (Cm_q-c3.Cm_coeff(3)) / Cm_q * 100);
fprintf('\t$C_{m_{\\delta_e}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_delta_e, c3.Cm_coeff(4), (Cm_delta_e-c3.Cm_coeff(4)) / Cm_delta_e * 100);
fprintf('\t$C_{n_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_0, c3.Cn_coeff(1), (Cn_0-c3.Cn_coeff(1)) / Cn_0 * 100);
fprintf('\t$C_{n_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_beta, c3.Cn_coeff(2), (Cn_beta-c3.Cn_coeff(2)) / Cn_beta * 100);
fprintf('\t$C_{n_{r}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_r, c3.Cn_coeff(3), (Cn_r-c3.Cn_coeff(3)) / Cn_r * 100);
fprintf('\t$C_{n_{\\delta_r}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_delta_r, c3.Cn_coeff(4), (Cn_delta_r-c3.Cn_coeff(4)) / Cn_delta_r * 100);

%Print coefficients of determination:
fprintf('Coefficients of determination:\n');
fprintf('\t%.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%%\\\\\n', c3.cod(1), c3.cod(2), c3.cod(3), c3.cod(4), c3.cod(5), c3.cod(6));
