%case4.m -  System identification using filtered values for alpha,
%           beta and q_bar.

%Create sensor signals to use in calculation:
c4.t = f.t_imu;
t = c4.t;

%Dynamic pressure sensor (estimated values):
c4.q_bar = f.q_bar_hat;

%Accelerometer:
c4.ax = sensor_a.signals.values(:, 1);
c4.ay = sensor_a.signals.values(:, 2);
c4.az = sensor_a.signals.values(:, 3);

%Angular velocity (gyro):
c4.p = sensor_omega.signals.values(:, 1);
c4.q = sensor_omega.signals.values(:, 2);
c4.r = sensor_omega.signals.values(:, 3);

%Magnetic field sensor:
c4.mx = sensor_m.signals.values(:, 1);
c4.my = sensor_m.signals.values(:, 2);
c4.mz = sensor_m.signals.values(:, 3);

%Delta controls:
c4.delta_a = sensor_delta.signals.values(:, 1);
c4.delta_e = sensor_delta.signals.values(:, 2);
c4.delta_r = sensor_delta.signals.values(:, 3);

%Angular acceleration:
c4.dp = mpoly(c4.p, t, 1, 1, 1, 1, t);
c4.dq = mpoly(c4.q, t, 2, 5, 5, 1, t);
c4.dr = mpoly(c4.r, t, 1, 1, 1, 1, t);
%c4.dp = [0; diff(c4.p) ./ diff(t)];
%c4.dq = [0; diff(c4.q) ./ diff(t)];
%c4.dr = [0; diff(c4.r) ./ diff(t)];

%Wind angles (from filtering results):
c4.alpha = f.alpha_hat;
c4.beta = f.beta_hat;

%Calculate body-axis coefficients:
c4.CX_mes = (m*c4.ax(:) - T) ./ (c4.q_bar*S);
c4.CY_mes = m*c4.ay(:) ./ (c4.q_bar*S);
c4.CZ_mes = m*c4.az(:) ./ (c4.q_bar*S);
%Find stability axis coefficients:
c4.CDs_mes = c4.CX_mes .* cos(c4.alpha) + c4.CZ_mes .*sin(c4.alpha);
c4.CYs_mes = c4.CY_mes;
c4.CLs_mes = c4.CZ_mes .* cos(c4.alpha) - c4.CX_mes .*sin(c4.alpha);
%Find wind axis coefficients:
c4.CDw_mes = c4.CDs_mes .* cos(c4.beta) + c4.CYs_mes .* sin(c4.beta);
c4.CYw_mes = c4.CYs_mes .* cos(c4.beta) - c4.CDs_mes .* sin(c4.beta);
c4.CLw_mes = c4.CLs_mes;
%Calculate angular coefficients:
c4.Cl_mes = (Jx*c4.dp - Jxz*(c4.p.*c4.q + c4.dr) + (Jz-Jy)*c4.q.*c4.r) ./ (c4.q_bar*S*b);
c4.Cm_mes = (Jy*c4.dq + (Jx-Jz)*c4.p.*c4.r + Jxz*(c4.p.*c4.p - c4.r.*c4.r)) ./ (c4.q_bar*S*c);
c4.Cn_mes = (Jz*c4.dr - Jxz*(c4.dp - c4.q.*c4.r) + (Jy-Jx)*c4.p.*c4.q) ./ (c4.q_bar*S*b);

%Find least square estimates:
[c4.CL_coeff c4.CL_phi] = lse(c4.CLw_mes, [c4.alpha]);
[c4.CD_coeff c4.CD_phi] = lse(c4.CDw_mes, [abs(c4.alpha), abs(c4.beta)]);
[c4.CC_coeff c4.CC_phi] = lse(c4.CYw_mes, [c4.beta]);
[c4.Cl_coeff c4.Cl_phi] = lse(c4.Cl_mes, [c4.beta, c4.p, c4.delta_a]);
[c4.Cm_coeff c4.Cm_phi] = lse(c4.Cm_mes, [c4.alpha, c4.q, c4.delta_e]);
[c4.Cn_coeff c4.Cn_phi] = lse(c4.Cn_mes, [c4.beta, c4.r, c4.delta_r]);

%Calculate cod relative simulator:
c4.cod(1) = cod(c4.CDw_mes, c4.CD_phi*c4.CD_coeff);
c4.cod(2) = cod(c4.CYw_mes, c4.CC_phi*c4.CC_coeff);
c4.cod(3) = cod(c4.CLw_mes, c4.CL_phi*c4.CL_coeff);
c4.cod(4) = cod(c4.Cl_mes, c4.Cl_phi*c4.Cl_coeff);
c4.cod(5) = cod(c4.Cm_mes, c4.Cm_phi*c4.Cm_coeff);
c4.cod(6) = cod(c4.Cn_mes, c4.Cn_phi*c4.Cn_coeff);

%Calculate cod after smoothing measured signals:
c4.CDw_mes_smooth = mpoly(c4.CDw_mes, t, 2, 10, 10, 0, t);
c4.CYw_mes_smooth = mpoly(c4.CYw_mes, t, 2, 10, 10, 0, t);
c4.CLw_mes_smooth = mpoly(c4.CLw_mes, t, 2, 10, 10, 0, t);
c4.Cl_mes_smooth = mpoly(c4.Cl_mes, t, 2, 10, 10, 0, t);
c4.Cm_mes_smooth = mpoly(c4.Cm_mes, t, 2, 10, 10, 0, t);
c4.Cn_mes_smooth = mpoly(c4.Cn_mes, t, 2, 10, 10, 0, t);
c4.cod_c(1) = cod(c4.CDw_mes_smooth, c4.CD_phi*c4.CD_coeff);
c4.cod_c(2) = cod(c4.CYw_mes_smooth, c4.CC_phi*c4.CC_coeff);
c4.cod_c(3) = cod(c4.CLw_mes_smooth, c4.CL_phi*c4.CL_coeff);
c4.cod_c(4) = cod(c4.Cl_mes_smooth, c4.Cl_phi*c4.Cl_coeff);
c4.cod_c(5) = cod(c4.Cm_mes_smooth, c4.Cm_phi*c4.Cm_coeff);
c4.cod_c(6) = cod(c4.Cn_mes_smooth, c4.Cn_phi*c4.Cn_coeff);


%Plotting:

%Plot aerodynamical forces:
figure(1);
clf;
subplot(3, 1, 1);
h = plot(c4.t, c4.CDw_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c4.t, c4.CD_phi*c4.CD_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(CD.time, CD.signals.values, 'g--');
set(h, 'LineWidth', 2);
title('Calculated, fitted and true aerodynamical forces', 'FontSize', 14);
ylabel('C_D');
legend( 'Calculated', 'Fitted', 'True', 'location', 'SouthEast', 'Orientation', 'horizontal');

subplot(3, 1, 2);
h = plot(c4.t, c4.CYw_mes, 'r');
hold on;
set(h, 'LineWidth', 1);
h = plot(c4.t, c4.CC_phi*c4.CC_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(CC.time, CC.signals.values, 'g--');
set(h, 'LineWidth', 2);
ylabel('C_C');

subplot(3, 1, 3);
h = plot(c4.t, c4.CLw_mes, 'r');
set(h, 'LineWidth', 1);
hold on;
h = plot(c4.t, c4.CL_phi*c4.CL_coeff, 'b--');
set(h, 'LineWidth', 3);
h = plot(CL.time, CL.signals.values, 'g--');
set(h, 'LineWidth', 2);
ylabel('C_L');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c4_aero_f.pdf');


%Plot aerodynamical moments:
figure(2);
clf;
subplot(3, 1, 1);
h = plot(c4.t, c4.Cl_phi*c4.Cl_coeff, 'b--');

set(h, 'LineWidth', 3);
hold on;
h = plot(Cl.time, Cl.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c4.t, c4.Cl_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_l');
legend('Fitted', 'True', 'Calculated', 'location', 'NorthEast', 'Orientation', 'horizontal');
title('Fitted, calculated and true aerodynamical moments', 'FontSize', 14);

subplot(3, 1, 2);
h = plot(c4.t, c4.Cm_phi*c4.Cm_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(Cm.time, Cm.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c4.t, c4.Cm_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_m');

subplot(3, 1, 3);
h = plot(c4.t, c4.Cn_phi*c4.Cn_coeff, 'b--');
set(h, 'LineWidth', 3);
hold on;
h = plot(Cn.time, Cn.signals.values, 'g--');
set(h, 'LineWidth', 2);
h = plot(c4.t, c4.Cn_mes, 'r');
set(h, 'LineWidth', 1);
ylabel('C_n');
xlabel('Time (s)');

print('-dpdf', '../report/gfx/c4_aero_m.pdf');

%Print table of results:
fprintf('Table of results:\n');
fprintf('\t$C_{D_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_0, c4.CD_coeff(1), (CD_0-c4.CD_coeff(1)) / CD_0 * 100);
fprintf('\t$C_{D_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_alpha, c4.CD_coeff(2), (CD_alpha-c4.CD_coeff(2)) / CD_alpha * 100);
fprintf('\t$C_{D_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CD_beta, c4.CD_coeff(3), (CD_beta-c4.CD_coeff(3)) / CD_beta * 100);
fprintf('\t$C_{C_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CC_0, c4.CC_coeff(1), (CC_0-c4.CC_coeff(1)) / CC_0 * 100);
fprintf('\t$C_{C_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CC_beta, c4.CC_coeff(2), (CC_beta-c4.CC_coeff(2)) / CC_beta * 100);
fprintf('\t$C_{L_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CL_0, c4.CL_coeff(1), (CL_0-c4.CL_coeff(1)) / CL_0 * 100);
fprintf('\t$C_{L_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', CL_alpha, c4.CL_coeff(2), (CL_alpha-c4.CL_coeff(2)) / CL_alpha * 100);
fprintf('\t$C_{l_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_0, c4.Cl_coeff(1), (Cl_0-c4.Cl_coeff(1)) / Cl_0 * 100);
fprintf('\t$C_{l_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_beta, c4.Cl_coeff(2), (Cl_beta-c4.Cl_coeff(2)) / Cl_beta * 100);
fprintf('\t$C_{l_{p}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_p, c4.Cl_coeff(3), (Cl_p-c4.Cl_coeff(3)) / Cl_p * 100);
fprintf('\t$C_{l_{\\delta_a}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cl_delta_a, c4.Cl_coeff(4), (Cl_delta_a-c4.Cl_coeff(4)) / Cl_delta_a * 100);
fprintf('\t$C_{m_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_0, c4.Cm_coeff(1), (Cm_0-c4.Cm_coeff(1)) / Cm_0 * 100);
fprintf('\t$C_{m_{\\alpha}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_alpha, c4.Cm_coeff(2), (Cm_alpha-c4.Cm_coeff(2)) / Cm_alpha * 100);
fprintf('\t$C_{m_{q}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_q, c4.Cm_coeff(3), (Cm_q-c4.Cm_coeff(3)) / Cm_q * 100);
fprintf('\t$C_{m_{\\delta_e}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cm_delta_e, c4.Cm_coeff(4), (Cm_delta_e-c4.Cm_coeff(4)) / Cm_delta_e * 100);
fprintf('\t$C_{n_0}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_0, c4.Cn_coeff(1), (Cn_0-c4.Cn_coeff(1)) / Cn_0 * 100);
fprintf('\t$C_{n_{\\beta}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_beta, c4.Cn_coeff(2), (Cn_beta-c4.Cn_coeff(2)) / Cn_beta * 100);
fprintf('\t$C_{n_{r}}$&\t\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_r, c4.Cn_coeff(3), (Cn_r-c4.Cn_coeff(3)) / Cn_r * 100);
fprintf('\t$C_{n_{\\delta_r}}$&\t%.3f & %.3f & %.1f\\%%\\\\\\hline\n', Cn_delta_r, c4.Cn_coeff(4), (Cn_delta_r-c4.Cn_coeff(4)) / Cn_delta_r * 100);

%Print coefficients of determination:
fprintf('Coefficients of determination:\n');
fprintf('\t%.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%%\\\\\n', c4.cod(1), c4.cod(2), c4.cod(3), c4.cod(4), c4.cod(5), c4.cod(6));
fprintf('\t%.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%% & %.2f\\%%\\\\\n', c4.cod_c(1), c4.cod_c(2), c4.cod_c(3), c4.cod_c(4), c4.cod_c(5), c4.cod_c(6));


