%==========================================================
% hua_update.m - Update filter state after a specified 
%                time dtwhen a velocity measurement from 
%                GPS is available
% x - filter state structure
% k_1 - filter parameter
% k_v - filter parameter
% a - accelration from an accelerometer
% vel - velocity measurement from GPS.
% dt - time step since last update.
%==========================================================
function [x] = hua_update(x, k_1, k_v, vel, a, dt)
%Find error between current estimate and measurement:
verr = vel - x.v_hat;

%Update v_hat:
x.v_hat = x.v_hat + dt * k_1 * verr;

%Update Q:
x.Q_hat = x.Q_hat + dt* k_v * verr * a';

%Normalize Q:
x.Q_hat = x.Q_hat / norm(x.Q_hat, 'fro') * sqrt(3);