
%Body initial conditions:
p_init = [0 0 0]';
v_body_init = [20 0 0]';
euler_init = [0 0 0]';
omega_init = [0 0 0]';

%Initial conditions precalc:
quat_init = e2q(euler_init);

%Aircraft settings:
m = 2.657;
Jx = 0.157;
Jy = 0.158;
Jz = 0.275;
Jxz = 0.000;
S = 0.348; %Wing area
l = 1.0; %Characteristic length
b = 1.58; %Wing span
c = 0.22; %Wing cord

%Aerodynamical parameters:
%global  CL_0 CL_alpha CD_0 CD_alpha CD_beta CC_beta ...
%        Cl_beta Cl_delta_a Cl_p Cm_alpha Cm_delta_e Cm_q ...
%        Cn_beta Cn_delta_r Cn_r
    
CL_0        = -0.2;
CL_alpha    = -0.9;
CD_0        = -0.15;
CD_alpha    = -0.3;
CD_beta     = -0.4;
CC_0        = 0.00;
CC_beta     = 0.1;
Cl_0        = 0.00;
Cl_beta     = -0.05;
Cl_delta_a  = -0.25;
Cl_p        = -0.02;
Cm_0        = 0.00;
Cm_alpha    = -0.2;
Cm_delta_e  = -0.15;
Cm_q        = -0.01;
Cn_0        = 0.00;
Cn_beta     = 0.12;
Cn_delta_r  = -0.21;
Cn_r        = -0.02;

%Sensor settings:
imu_ts = 0.01;  %IMU sampling time.
gps_ts = 0.1; %GPS sampling time.

% (ADIS16405 Specifications:)
gyro_initial_bias = [0 0 0]';
gyro_drift_noise_power = 0.000;

gps_initial_bias = [0 0 0]';
gps_drift_noise_power = 0.001;

gps_vel_noise_density = 0.033; %m/sqrt(Hz) rms
gyro_noise_density = 3 * 0.05 * (pi/180.0); %rad/sqrt(Hz) rms
acc_noise_density = 3 * 0.5 * (0.001*9.81); %m/s/sqrt(Hz) rms
mag_noise_density = 3 * 0.066 * (0.001); %gauss/sqrt(Hz) rms
vane_noise_density = 0.05 * (pi/180.0); %rad/sqrt(Hz) rms
pitot_noise_density = 2; %Pa/sqrt(Hz) rms

% Band limited white noise power:
gps_noise_power = gps_noise_density^2;
gps_vel_noise_power = gps_vel_noise_density^2;
gyro_noise_power = gyro_noise_density^2;
acc_noise_power = acc_noise_density^2;
mag_noise_power = mag_noise_density^2;
vane_noise_power = vane_noise_density^2;
pitot_noise_power = pitot_noise_density^2;

%Vane positions:
alpha_vane_position = [0 -b/2-2*c 0]';
beta_vane_position = [7/3*c -1/4*b 0]';

%Inertia precalc:
J = [Jx 0 -Jxz; 0 Jy 0; -Jxz 0 Jz];
J_inv = inv(J);

%Thrust configuration:
T = 10; %N
thrust_f = T*[1 0 0]'; %Thrust force vector
